@@ -14,9 +14,6 @@ formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(messag
 fh.setFormatter(formatter)
 logger.addHandler(fh)
 
-# dtwalign: https://github.com/statefb/dtwalign
-# from dtwalign import dtw_from_distance_matrix
-
 epsqrt = np.sqrt(np.finfo(float).eps)
 
 
@@ -65,7 +62,7 @@ def determine_scaling(
 
         rnge[rnge.values == 0] = 1.0
         collector_rnge.append(rnge)
-        collector_mins.append(batch.min())
+        collector_mins.append(batch.min(axis=0))
 
     if settings["robust"]:
         scalings = pd.concat(
@@ -81,7 +78,7 @@ def determine_scaling(
             axis=1,
         )
     scalings.columns = ["Range", "Minimum"]
-    scalings["Minimum"] = 0.0
+    # scalings["Minimum"] = 0.0
     return scalings
 
 
@@ -165,21 +162,23 @@ class DTWresult:
 def align_with_path(md_path, batch, initial_row):
     row = 0
     nr = md_path[:, 0].max() + 1  # to account for the zero-based indexing
-    synced = np.zeros((nr, batch.shape[1]))
-    synced[row, :] = batch.iloc[md_path[0, 1], :]
+    synced = pd.DataFrame(np.zeros((nr, batch.shape[1])), columns=batch.columns)
+    synced.iloc[row, :] = batch.iloc[md_path[0, 1], :]
     temp = initial_row
     for idx in np.arange(1, md_path.shape[0]):
         if md_path[idx, 0] != md_path[idx - 1, 0]:
             row += 1
-            synced[row, :] = temp = batch.iloc[md_path[idx, 1], :]
+            synced.iloc[row, :] = temp = batch.iloc[md_path[idx, 1], :]
 
         else:
             # TODO : Come back to page 181 of thesis: where more than 1 point in the target
             #        trajectory is aligned with the reference: compute the average,
             temp = np.vstack((temp, batch.iloc[md_path[idx, 1], :]))
-            synced[row, :] = np.nanmean(temp, axis=0)
+            synced.iloc[row, :] = np.nanmean(temp, axis=0)
 
-    return pd.DataFrame(synced, columns=batch.columns)
+    return pd.DataFrame(
+        synced,
+    )
 
 
 def dtw_core(test, ref, weight_matrix: np.ndarray):
@@ -334,7 +333,7 @@ def batch_dtw(
     weight_vector = np.ones(refbatch_sc.shape[1])
     weight_matrix = np.diag(weight_vector)
     weight_history = np.zeros_like(weight_vector) * np.nan
-
+    average_batch = None
     delta_weight = np.linalg.norm(weight_vector)
     iter = 0
     while (np.linalg.norm(delta_weight) > settings["tolerance"]) and (
@@ -349,6 +348,9 @@ def batch_dtw(
         weight_matrix = np.diag(weight_vector)
         weight_history = np.vstack((weight_history, weight_vector.copy()))
 
+        if iter > 3:
+            refbatch_sc = average_batch
+
         aligned_batches, average_batch = one_iteration_dtw(
             batches_scaled=batches_scaled,
             refbatch_sc=refbatch_sc,
@@ -358,7 +360,7 @@ def batch_dtw(
 
         # Deviations from the average batch:
         next_weights = np.zeros((1, refbatch_sc.shape[1]))
-        for _, result in aligned_batches.items():
+        for batch_id, result in aligned_batches.items():
             next_weights += np.nansum(
                 np.power(result.synced - average_batch, 2), axis=0
             )
@@ -384,6 +386,7 @@ def batch_dtw(
     # scaling for the trajectories
     weight_history = weight_history[1:, :]
     aligned_df = pd.DataFrame()
+
     for batch_id, result in aligned_batches.items():
         initial_row = batches[batch_id].iloc[result.md_path[0, 0], :].copy()
         synced = align_with_path(
@@ -391,7 +394,11 @@ def batch_dtw(
             batches[batch_id].iloc[:: int(settings["subsample"]), :],
             initial_row=initial_row,
         )
+        if "batch_id" not in synced.columns:
+            synced.insert(0, "batch_id", batch_id)
         synced.insert(1, "sequence", list(range(synced.shape[0])))
+
+        result.synced = synced  # overwrite existing dataframe with this, unscaled, one.
         aligned_df = aligned_df.append(synced)
 
     max_places = int(np.ceil(np.log10(aligned_df["sequence"].max())))
@@ -413,6 +420,6 @@ def batch_dtw(
         scale_df=scale_df,
         aligned_batch_objects=aligned_batches,
         last_average_batch=last_average_batch,
-        weight_history=weight_history,
+        weight_history=pd.DataFrame(weight_history, columns=columns_to_align),
         aligned_wide_df=aligned_wide_df,
     )
